# --------------------------------------------------------------------
# --   *****************************
# --   *   Trenz Electronic GmbH   *
# --   *   Holzweg 19A             *
# --   *   32257 Bünde             *
# --   *   Germany                 *
# --   *****************************
# --------------------------------------------------------------------
# -- $Author :  Pohl, Zdenek $
# -- $Email :   zdenek.pohl@utia.cas.cz$
# --------------------------------------------------------------------
# -- Descriptio:
# --    TCL script is used to export sdsoc platform using xsct
# -- Change History:
# ------------------------------------------
# -- $Date: 2018/02/03 | $Author: Pohl, Zdenek
# -- - initial release
# -- - todo - support also Zynq platform and ZynqMP with libraries 
# ------------------------------------------

#load parameters saved by export script to be used in sdsoc export process
source pfm_settings.tcl

# create hardware platform 
platform -name ${design_name}\
         -desc "Basic Trenz TE0701 + TE0720 Zynq platform" \
         -hw "${tmp_path}/hw/${design_name}.dsa" \
         -out "${tmp_path}/PF" \
         -samples "${tmp_path}/samples" \
         -prebuilt


#search if library files for platform are provided
#we support one library file for standalone and one for linux targes
set standalone_libs [glob -nocomplain -directory ${tmp_path}/sw/${standalone_path}/lib *.a]
set linux_libs [glob -nocomplain -directory ${tmp_path}/sw/${linux_path}/lib *.a] 

 if {[llength $standalone_libs] == 0 && 
     [llength $linux_libs] == 0} {

     system -name standalone\
         -display-name "Standalone" \
         -desc {} \
         -boot "${tmp_path}/sw/${standalone_path}/boot" \
         -readme "${tmp_path}/sw/${standalone_path}/boot/generic.readme"

     boot -bif "${tmp_path}/sw/${standalone_path}/boot/standalone.bif"

     domain -name standalone \
       -proc ps7_cortexa9_0 \
       -display-name "Standalone" \
       -desc {} \
       -os {standalone} \
       -runtime {cpp}

     domain -prebuilt-data "${tmp_path}/prebuilt"

     app -lscript "${tmp_path}/sw/${standalone_path}/lscript.ld"

     system -name {Linux}\
       -display-name {Linux} \
       -desc {} \
       -boot "${tmp_path}/sw/${linux_path}/boot"

     boot -bif "${tmp_path}/sw/${linux_path}/boot/linux.bif"

     domain -name {Linux} \
       -os {linux} \
       -proc {ps7_cortexa9_0} \
       -display-name {Linux} \
       -desc {} \
       -runtime {cpp} \
       -image "${tmp_path}/sw/${linux_path}/image"

     domain -prebuilt-data "${tmp_path}/prebuilt"
 }

#generate platform
platform -generate 


