/***************************************************************************

*   © Copyright 2013 Xilinx, Inc. All rights reserved. 

*   This file contains confidential and proprietary information of Xilinx,
*   Inc. and is protected under U.S. and international copyright and other
*   intellectual property laws. 

*   DISCLAIMER
*   This disclaimer is not a license and does not grant any rights to the
*   materials distributed herewith. Except as otherwise provided in a valid
*   license issued to you by Xilinx, and to the maximum extent permitted by
*   applicable law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH
*   ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES AND CONDITIONS, 
*   EXPRESS, IMPLIED, OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES
*   OF MERCHANTABILITY, NON-INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR 
*   PURPOSE; and (2) Xilinx shall not be liable (whether in contract or 
*   tort, including negligence, or under any other theory of liability)
*   for any loss or damage of any kind or nature related to, arising under
*   or in connection with these materials, including for any direct, or any
*   indirect, special, incidental, or consequential loss or damage (including
*   loss of data, profits, goodwill, or any type of loss or damage suffered 
*   as a result of any action brought by a third party) even if such damage
*   or loss was reasonably foreseeable or Xilinx had been advised of the 
*   possibility of the same. 

*   CRITICAL APPLICATIONS 
*   Xilinx products are not designed or intended to be fail-safe, or for use
*   in any application requiring fail-safe performance, such as life-support
*   or safety devices or systems, Class III medical devices, nuclear facilities,
*   applications related to the deployment of airbags, or any other applications
*   that could lead to death, personal injury, or severe property or environmental
*   damage (individually and collectively, "Critical Applications"). Customer
*   assumes the sole risk and liability of any use of Xilinx products in Critical
*   Applications, subject only to applicable laws and regulations governing 
*   limitations on product liability. 

*   THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT 
*   ALL TIMES.

***************************************************************************/

#define HLS_NO_XIL_FPO_LIB
#include "hls_video.h"
#include "top.h"
#include "transform.h"

void readLRinput (yuv_t *inLR,
	    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1>& img_l,
	    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1>& img_r,
		int height, int dual_width, int width, int stride)
{
  for (int i=0; i < height; ++i) {
#pragma HLS loop_tripcount min=1080 max=1080 avg=1080
    for (int j=0; j < stride; ++j) {
#pragma HLS loop_tripcount min=1920 max=1920 avg=1920
    #pragma HLS PIPELINE
      yuv_t tmpData = inLR [i*stride + j];	// from yuv_t array: consume height*stride
      if (j < width)
    	  img_l.write (tmpData & 0x00FF);	// to HLS_8UC1 stream
      else if (j < dual_width)
    	  img_r.write (tmpData & 0x00FF);	// to HLS_8UC1 stream
    }
  }
}

void writeDispOut(hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1>& img_d,
					yuv_t *dst,
					int height, int width, int stride)
{
  pix_t tmpOut;
  yuv_t outData;

  for (int i=0; i < height; ++i) {
#pragma HLS loop_tripcount min=1080 max=1080 avg=1080
    for (int j=0; j < stride; ++j) {
#pragma HLS loop_tripcount min=960 max=960 avg=960
    #pragma HLS PIPELINE
	  if (j < width) {
		tmpOut = img_d.read().val[0];
		outData = ((yuv_t) 0x8000) | ((yuv_t)tmpOut);
		dst [i*stride +j] = outData;
	  }
	  else {
		outData = (yuv_t) 0x8000;
		dst [i*stride +j] = outData;
	  }
    }
  }
}

namespace hls {
void SaveAsGray(
            Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16SC1>& src,
            Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1>& dst)
{
    int height = src.rows;
    int width = src.cols;
    //short smin = (short) 0x7FFF; // biggest signed 16bit
    //short smax = (short) 0x8000; // smallest signed 16bit
    //int range;

    for (int i = 0; i < height; i++) {
#pragma HLS loop_tripcount min=1080 max=1080 avg=1080
        for (int j = 0; j < width; j++) {
#pragma HLS loop_tripcount min=960 max=960 avg=960
#pragma HLS pipeline II=1
            Scalar<1, short> s;
            Scalar<1, unsigned char> d;
            src >> s;
            //smin = (s.val[0] < smin)? s.val[0] : smin;
            //smax = (s.val[0] > smax)? s.val[0] : smax;
            int oval = (int)s.val[0] + 16;
// WSIZE=15, NDISP  32: min=-16, max= 496, range= 512 (2^9)
// WSIZE=15, NDISP 128: min=-16, max=2032, range=2048 (2^11)
            if (oval < 0) oval = 0;
            if (oval > 511) oval = 511;
            d.val[0] = (unsigned char)(oval >> 1);
            dst << d;
        }
    }
    //range = (int)smax - (int)smin;
    //printf ("min=%d max=%d range=%d\n",smin,smax,range);
}
} // namespace hls

int stereo_remap_bm_new(
        yuv_t *img_data_lr,
        yuv_t *img_data_disp,
        hls::Window<3, 3, param_T > &lcameraMA_l,
        hls::Window<3, 3, param_T > &lcameraMA_r,
        hls::Window<3, 3, param_T > &lirA_l,
        hls::Window<3, 3, param_T > &lirA_r,
        param_T (&ldistC_l)[5],
        param_T (&ldistC_r)[5],
        int height,		// 1080
        int dual_width,	// 1920 (two 960x1080 images side by side)
        int stride_in,	// 1920 (two 960x1080 images side by side)
        int stride_out)	// 960
{
	int width = dual_width/2; // 960

#pragma HLS DATAFLOW

    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1> img_l(height, width);
#pragma HLS STREAM variable=img_l.data_stream depth=960
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1> img_r(height, width);

    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1> img_l_remap(height, width);	// remapped left image
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1> img_r_remap(height, width);	// remapped left image
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_8UC1> img_d(height, width);

    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16SC2> map1_l(height, width);
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16SC2> map1_r(height, width);
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16UC2> map2_l(height, width);
    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16UC2> map2_r(height, width);

    hls::Mat<IMG_HEIGHT, IMG_WIDTH, HLS_16SC1> img_disp(height, width);
    hls::StereoBMState<WSIZE, NDISP, NDISP_UNIT> state;

  // ddr -> kernel streams: extract luma from left and right yuv images
  // store it in single channel HLS_8UC1 left and right Mat's
	readLRinput (img_data_lr, img_l, img_r, height, dual_width, width, stride_in);

  //////////////////////// remap left and right images, all types are HLS_8UC1 //////////
    hls::InitUndistortRectifyMapInverse(lcameraMA_l, ldistC_l, lirA_l, map1_l, map2_l);
    hls::Remap<8>(img_l, img_l_remap, map1_l, map2_l, HLS_INTER_LINEAR);
    hls::InitUndistortRectifyMapInverse(lcameraMA_r, ldistC_r, lirA_r, map1_r, map2_r);
    hls::Remap<8>(img_r, img_r_remap, map1_r, map2_r, HLS_INTER_LINEAR);

  ////////// find disparity of remapped images //////////
    hls::FindStereoCorrespondenceBM(img_l_remap, img_r_remap, img_disp, state);
    hls::SaveAsGray(img_disp, img_d);

	// kernel stream -> ddr : output single wide
	writeDispOut (img_d, img_data_disp, height, width, stride_out);

	return 0;
}

int stereo_remap_bm(
        yuv_t *img_data_lr,
        yuv_t *img_data_disp,
        int height,		// 1080
        int dual_width,	// 1920 (two 960x1080 images side by side)
        int stride_in,	// 1920 (two 960x1080 images side by side)
        int stride_out)	// 960
{
//1920*1080
//#pragma HLS interface m_axi port=img_data_lr depth=2073600
//#pragma HLS interface m_axi port=img_data_disp depth=2073600

    hls::Window<3, 3, param_T > lcameraMA_l;
    hls::Window<3, 3, param_T > lcameraMA_r;
    hls::Window<3, 3, param_T > lirA_l;
    hls::Window<3, 3, param_T > lirA_r;
    param_T ldistC_l[5];
    param_T ldistC_r[5];

    for (int i=0; i<3; i++) {
        for (int j=0; j<3; j++) {
            lcameraMA_l.val[i][j]=cameraMA_l[i*3+j];
            lcameraMA_r.val[i][j]=cameraMA_r[i*3+j];
            lirA_l.val[i][j]=irA_l[i*3+j];
            lirA_r.val[i][j]=irA_r[i*3+j];
        }
    }
    for (int i=0; i<5; i++) {
        ldistC_l[i] = distC_l[i];
        ldistC_r[i] = distC_r[i];
    }

    int ret = stereo_remap_bm_new(img_data_lr,
                                img_data_disp,
                                lcameraMA_l,
                                lcameraMA_r,
                                lirA_l,
                                lirA_r,
                                ldistC_l,
                                ldistC_r,
                                height,
                                dual_width,
                                stride_in,
                                stride_out);
    return ret;
}

